from django.db import migrations


class Migration(migrations.Migration):
    operations = [
        migrations.RunSQL("""
        do $$ begin

        CREATE TABLE queue (
          id          bigserial    PRIMARY KEY,
          enqueued_at timestamptz  NOT NULL DEFAULT current_timestamp,
          dequeued_at timestamptz,
          expected_at timestamptz,
          schedule_at timestamptz,
          q_name      text         NOT NULL CHECK (length(q_name) > 0),
          data        json         NOT NULL
        );

        end $$ language plpgsql;

        create index priority_idx on queue
        (schedule_at nulls first, expected_at nulls first)
        where dequeued_at is null;

        drop function if exists pq_notify() cascade;

        create function pq_notify() returns trigger as $$ begin
          perform pg_notify(new.q_name, '');
          return null;
        end $$ language plpgsql;

        create trigger pq_insert
        after insert on queue
        for each row
        execute procedure pq_notify();
        """),
    ]



from django.core.management.base import BaseCommand

from dj_pq import get_queue


class Command(BaseCommand):
    help = 'Starts new worker'

    def add_arguments(self, parser):
        parser.add_argument('queue_name', type=str)
        parser.add_argument(
            '--burst',
            action='store_true',
            default=False,
        )

    def handle(self, **options):
        name = options['queue_name']
        burst = options['burst']
        get_queue(name).work(burst)



from django.test import TestCase

from pq.tasks import Queue
from dj_pq import get_queue


class DjPqTest(TestCase):
    def test_get_queue(self):
        queue = get_queue('notifications')
        self.assertTrue(isinstance(queue, Queue))

    def test_get_queue_twice(self):
        queue = get_queue('notifications')
        self.assertIs(
            queue,
            get_queue('notifications'),
        )

    def test_get_queue_two_databases(self):
        queue = get_queue('notifications')
        self.assertIsNot(
            queue,
            get_queue('notifications', connection_name='another'),
        )

    def test_enqueue(self):
        queue = get_queue('notifications')

        global test_value
        test_value = 40

        @queue.task()
        def task(value):
            global test_value
            test_value += value

        task(2)

        self.assertEqual(test_value, 40)
        self.assertTrue(queue.perform(queue.get()))
        self.assertEqual(test_value, 42)

        del test_value



from django.db import connections

from pq.tasks import PQ, Queue


_pqs = dict()


def get_queue(name, connection_name='default'):
    try:
        return _pqs[connection_name][name]

    except KeyError:
        return _pqs.setdefault(connection_name, PQ(
            connections[connection_name].cursor().connection,
            queue_class=Queue,
        ))[name]



from setuptools import setup, find_packages


setup(
    name='dj-pq',
    version='1.4.1',
    license='BSD License',
    author='jean-philippe serafin',
    url='https://github.com/jeanphix/dj-pq/',
    author_email='serafinjp@gmail.com',
    description='``pq`` wrapper for django',
    long_description=open('README.rst').read(),
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Internet',
        'Topic :: System :: Distributed Computing',
        'Topic :: System :: Systems Administration',
    ],
    install_requires=[
        'django',
        'pq==1.4',
    ],
)
